"use strict";
{
  const C3 = self.C3;
  C3.Behaviors.Rex_mode7perspective.Instance = class Rex_mode7perspectiveInstance extends (
    C3.SDKBehaviorInstanceBase
  ) {
    constructor(behInst, properties) {
      super(behInst);

      this.LX = 0;
      this.LY = 0;

      if (properties) {
        this.LX = properties[0];
        this.LY = properties[1];
      }

      this.default_width = this._inst._worldInfo.GetWidth();
      this.default_height = this._inst._worldInfo.GetHeight();

      this.ground_inst = null;
      this.ground_instUid = -1; // for loading

      // Opt-in to getting calls to Tick()
      this._StartTicking();
    }

    Release() {
      super.Release();
    }

    SaveToJson() {
      return {
        LX: this.LX,
        LY: this.LY,
        ground_uid: this.ground_inst ? this.ground_inst.GetUID() : -1,
      };
    }

    LoadFromJson(o) {
      this.LX = o["LX"];
      this.LY = o["LY"];
      this.ground_instUid = o["ground_uid"];
    }

    AfterLoad() {
      // Look up the pinned object UID now getObjectByUID is available
      if (this.ground_instUid === -1) this.ground_inst = null;
      else {
        this.ground_inst = this._runtime.GetInstanceByUID(this.ground_instUid);
        if (!this.ground_inst)
          console.error("Failed to find ground object by UID");
      }

      this.ground_instUid = -1;
    }

    Tick() {
      if (this.ground_inst === null && this.ground_instUid !== -1)
        this.AfterLoad();
      this.position_update(this.ground_inst);
    }

    GetDebuggerProperties() {
      return [
        {
          title: "Rex_mode7perspective",
          properties: [
            {
              name: "LX",
              value: this.LX,
              onedit: (v) => (this.LX = v),
            },
            {
              name: "LY",
              value: this.LY,
              onedit: (v) => (this.LY = v),
            },
          ],
        },
      ];
    }

    position_update(ground_inst) {
      if (!ground_inst) return;

      var mode7Ground_behavior =
        this.get_mode7Ground_behavior_inst(ground_inst);
      var result = mode7Ground_behavior.LXY2PXY(this.LX, this.LY);

      var needUpdatingPosition =
        typeof result.x === "number" &&
        typeof result.y === "number" &&
        (this._inst._worldInfo.GetX() !== result.x ||
          this._inst._worldInfo.GetY() !== result.y);

      if (needUpdatingPosition) {
        this._inst._worldInfo.SetX(result.x);
        this._inst._worldInfo.SetY(result.y);
        this._inst._worldInfo.SetBboxChanged();
        this._runtime.UpdateRender();
      }

      var new_width = this.default_width * result.scale;
      var new_height = this.default_height * result.scale;
      if (
        this._inst._worldInfo.GetWidth() !== new_width ||
        this._inst._worldInfo.GetHeight() !== new_height
      ) {
        this._inst._worldInfo.SetWidth(new_width);
        this._inst._worldInfo.SetHeight(new_height);
        this._inst._worldInfo.SetBboxChanged();
        this._runtime.UpdateRender();
      }

      // visible
      var visible = result.pz >= 0;
      if (this._inst._worldInfo.IsVisible() !== visible) {
        this._inst._worldInfo.SetVisible(visible);
        this._runtime.UpdateRender();
      }
    }

    get_mode7Ground_behavior_inst(ground_inst) {
      var has_mode7Ground_behavior = C3.Behaviors.Rex_mode7ground != null;
      if (!has_mode7Ground_behavior)
        console.error(
          "[Mode7 perspective] Could not find mode 7 ground behavior"
        );

      var behavior_inst;
      var i,
        cnt = ground_inst._behaviorInstances.length;
      for (i = 0; i < cnt; i++) {
        behavior_inst = ground_inst._behaviorInstances[i]._sdkInst;
        if (behavior_inst instanceof C3.Behaviors.Rex_mode7ground.Instance)
          return behavior_inst;
      }

      if (!behavior_inst)
        console.error(
          "[Mode7 perspective] Could not find mode 7 ground behavior"
        );
      return null;
    }
  };
}
